Stan Math Library: stan/math/prim/scal/fun/inc_beta_ddb.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

inc_beta_ddb.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_FUN_INC_BETA_DDB_HPP
2 #define STAN_MATH_PRIM_SCAL_FUN_INC_BETA_DDB_HPP
3
4 #include <stan/math/prim/scal/fun/inc_beta.hpp>
5 #include <stan/math/prim/scal/fun/inc_beta_dda.hpp>
6 #include <cmath>
7 #include <stdexcept>
8
9 namespace stan {
10 namespace math {
11
12 template <typename T>
13 T inc_beta_dda(T a, T b, T z,
14 T digamma_a, T digamma_ab);
15
38 template <typename T>
39 T inc_beta_ddb(T a, T b, T z,
40 T digamma_b, T digamma_ab) {
41 using std::log;
42
43 if (b > a)
44 if ((0.1 < z && z <= 0.75 && b > 500)
45 || (0.01 < z && z <= 0.1 && b > 2500)
46 || (0.001 < z && z <= 0.01 && b > 1e5))
47 return -inc_beta_dda(b, a, 1 - z, digamma_b, digamma_ab);
48
49 if ((z > 0.75 && a < 500)
50 || (z > 0.9 && a < 2500)
51 || (z > 0.99 && a < 1e5)
52 || (z > 0.999))
53 return -inc_beta_dda(b, a, 1 - z, digamma_b, digamma_ab);
54
55 double threshold = 1e-10;
56
57 // Common prefactor to regularize numerator and denomentator
58 T prefactor = (a + 1) / (a + b);
59 prefactor = prefactor * prefactor * prefactor;
60
61 T sum_numer = digamma_ab * prefactor;
62 T sum_denom = prefactor;
63
64 T summand = prefactor * z * (a + b) / (a + 1);
65
66 T k = 1;
67 digamma_ab += 1.0 / (a + b);
68
69 while (fabs(summand) > threshold) {
70 sum_numer += digamma_ab * summand;
71 sum_denom += summand;
72
73 summand *= (1 + (a + b) / k) * (1 + k) / (1 + (a + 1) / k);
74 digamma_ab += 1.0 / (a + b + k);
75 ++k;
76 summand *= z / k;
77
78 if (k > 1e5)
79 throw std::domain_error("stan::math::inc_beta_ddb did "
80 "not converge within 100000 iterations");
81 }
82
83 return inc_beta(a, b, z)
84 * (log(1 - z) - digamma_b + sum_numer / sum_denom);
85 }
86
87 } // math
88 } // stan
89
90 #endif
stan::math::fabs
fvar< T > fabs(const fvar< T > &x)
Definition: fabs.hpp:14
stan
Definition: log_sum_exp.hpp:8
stan::math::inc_beta_dda
T inc_beta_dda(T a, T b, T z, T digamma_a, T digamma_ab)
Returns the partial derivative of the regularized incomplete beta function, I_{z}(a, b) with respect to a.
Definition: inc_beta_dda.hpp:39
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::inc_beta_ddb
T inc_beta_ddb(T a, T b, T z, T digamma_b, T digamma_ab)
Returns the partial derivative of the regularized incomplete beta function, I_{z}(a, b) with respect to b.
Definition: inc_beta_ddb.hpp:39
stan::math::inc_beta
fvar< T > inc_beta(const fvar< T > &a, const fvar< T > &b, const fvar< T > &x)
Definition: inc_beta.hpp:20
stan::math::domain_error
void domain_error(const char *function, const char *name, const T &y, const char *msg1, const char *msg2)
Throw a domain error with a consistently formatted message.
Definition: domain_error.hpp:32
stan::math::e
double e()
Return the base of the natural logarithm.
Definition: constants.hpp:95
inc_beta.hpp
inc_beta_dda.hpp

[ Stan Home Page ] © 2011–2016, Stan Development Team.